"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Mapper = void 0;
/* eslint-disable @typescript-eslint/naming-convention, unicorn/expiring-todo-comments */
const RmlParser = __importStar(require("@comake/rmlmapper-js"));
const jsonld_1 = __importDefault(require("jsonld"));
const TripleUtil_1 = require("../util/TripleUtil");
const Vocabularies_1 = require("../util/Vocabularies");
class Mapper {
    constructor(args) {
        this.functions = args?.functions;
    }
    async apply(data, mapping, frame) {
        const result = await this.doMapping(data, mapping);
        return await this.frame(result, frame);
    }
    async applyAndFrameSklProperties(data, mapping, frame) {
        const result = await this.doMapping(data, mapping);
        return await this.frameSklPropertiesAndConvertToNativeTypes(result, frame);
    }
    async doMapping(data, mapping) {
        const mappingAsQuads = await this.jsonLdToQuads(mapping);
        const sources = { 'input.json': JSON.stringify(data) };
        const options = { functions: this.functions };
        // TODO always return arrays...
        return await RmlParser.parse(mappingAsQuads, sources, options);
    }
    async frame(jsonldDoc, overrideFrame) {
        let frame = {
            '@context': {},
            '@embed': '@always',
        };
        frame = {
            ...frame,
            ...overrideFrame,
            '@context': { ...frame['@context'], ...overrideFrame?.['@context'] },
        };
        return await jsonld_1.default.frame(jsonldDoc, frame);
    }
    async frameSklPropertiesAndConvertToNativeTypes(jsonldDoc, overrideFrame) {
        let frame = {
            '@context': {},
            '@embed': '@always',
        };
        this.addDefaultTopLevelContextWithSKLPropertiesAndConvertNativeValues(jsonldDoc, frame);
        frame = { ...frame, ...overrideFrame };
        return await jsonld_1.default.frame(jsonldDoc, frame);
    }
    addDefaultTopLevelContextWithSKLPropertiesAndConvertNativeValues(jsonldDoc, frame) {
        for (const subDoc of jsonldDoc) {
            Object.keys(subDoc).forEach((key) => {
                if (key.startsWith(Vocabularies_1.sklNamespace)) {
                    const argName = key.slice(Vocabularies_1.sklNamespace.length);
                    const value = subDoc[key];
                    if (Array.isArray(value) && typeof value[0] === 'object' && '@type' in value[0]) {
                        frame['@context'][argName] = { '@id': key, '@type': value[0]['@type'] };
                        subDoc[key] = subDoc[key].map((valueItem) => this.convertToNativeValue(valueItem));
                    }
                    else if (Array.isArray(value) && typeof value[0] === 'object' && '@id' in value[0]) {
                        frame['@context'][argName] = { '@id': key, '@type': '@id' };
                    }
                    else if (typeof value === 'object' && '@type' in value) {
                        frame['@context'][argName] = { '@id': key, '@type': value['@type'] };
                        subDoc[key] = this.convertToNativeValue(subDoc[key]);
                    }
                    else if (typeof value === 'object' && '@id' in value) {
                        frame['@context'][argName] = {
                            '@id': key,
                            '@type': '@id',
                        };
                    }
                    else {
                        frame['@context'][argName] = key;
                    }
                }
            });
        }
    }
    convertToNativeValue(jsonLdTerm) {
        return {
            ...jsonLdTerm,
            '@value': (0, TripleUtil_1.toJSValueFromDataType)(jsonLdTerm['@value'], jsonLdTerm['@type']),
        };
    }
    async jsonLdToQuads(jsonldDoc) {
        return (await jsonld_1.default.toRDF(jsonldDoc, { format: 'application/n-quads' }));
    }
}
exports.Mapper = Mapper;
//# sourceMappingURL=Mapper.js.map