import type { Triple, Expression } from 'sparqljs';
import type { Entity } from '../util/Types';
import type { QueryAdapter, FindOneOptions, FindAllOptions, FindOptionsWhere } from './QueryAdapter';
export interface WhereQueryData {
    triples: Triple[];
    filters: Expression[];
}
/**
 * A {@link QueryAdapter} that stores data in a database through a sparql endpoint.
 */
export declare class SparqlQueryAdapter implements QueryAdapter {
    private readonly sparqlClient;
    private readonly sparqlGenerator;
    constructor(endpointUrl: string);
    find(options?: FindOneOptions): Promise<Entity | null>;
    private buildPatternsFromQueryData;
    private createWhereQueryData;
    findBy(where: FindOptionsWhere): Promise<Entity | null>;
    findAll(options?: FindAllOptions): Promise<Entity[]>;
    findAllBy(where: FindOptionsWhere): Promise<Entity[]>;
    save(entity: Entity): Promise<Entity>;
    save(entities: Entity[]): Promise<Entity[]>;
    destroy(entity: Entity): Promise<Entity>;
    destroy(entities: Entity[]): Promise<Entity[]>;
    private executeSparqlQueryAndGetData;
    private valueToLiteral;
    private buildEqualOperation;
    private buildTypesAndSuperTypesTriple;
    private createWhereQueryDataFromKeyValue;
    private createWhereQueryDataForIdValue;
    private createWhereQueryDataForType;
    private createWhereQueryDataForNestedWhere;
    private createWhereQueryDataForLiteral;
}
