import type { GraphObject } from 'jsonld';
import type { Frame } from 'jsonld/jsonld-spec';
import type { Entity } from '../../util/Types';
import type { FindOneOptions, FindAllOptions, FindOptionsWhere } from '../FindOptionsTypes';
import type { QueryAdapter, RawQueryResult } from '../QueryAdapter';
import type { MemoryQueryAdapterOptions } from './MemoryQueryAdapterOptions';
/**
 * A {@link QueryAdapter} that stores data in memory.
 */
export declare class MemoryQueryAdapter implements QueryAdapter {
    private readonly schemas;
    private readonly setTimestamps;
    constructor(options: MemoryQueryAdapterOptions);
    executeRawQuery<T extends RawQueryResult>(query: string): Promise<T[]>;
    executeRawEntityQuery(query: string, frame?: Frame): Promise<GraphObject>;
    find(options?: FindOneOptions): Promise<Entity | null>;
    findBy(where: FindOptionsWhere): Promise<Entity | null>;
    findAll(options?: FindAllOptions): Promise<Entity[]>;
    findAllBy(where: FindOptionsWhere): Promise<Entity[]>;
    private entityMatchesQuery;
    private entityMatchesField;
    private fieldValueMatchesField;
    private findOptionWhereMatchesNodeObject;
    private handleOperator;
    private isInstanceOf;
    private getSubClassesOf;
    exists(where: FindOptionsWhere): Promise<boolean>;
    count(where?: FindOptionsWhere): Promise<number>;
    save(entity: Entity): Promise<Entity>;
    save(entities: Entity[]): Promise<Entity[]>;
    private saveEntity;
    destroy(entity: Entity): Promise<Entity>;
    destroy(entities: Entity[]): Promise<Entity[]>;
    destroyAll(): Promise<void>;
    private destroyEntity;
}
