import type { Literal, NamedNode } from '@rdfjs/types';
import type { GraphObject } from 'jsonld';
import type { Frame } from 'jsonld/jsonld-spec';
import type { Entity } from '../../util/Types';
import type { FindOneOptions, FindAllOptions, FindOptionsWhere } from '../FindOptionsTypes';
import type { QueryAdapter, RawQueryResult } from '../QueryAdapter';
import type { SparqlQueryAdapterOptions } from './SparqlQueryAdapterOptions';
export declare type SelectVariableQueryResult<T> = Record<keyof T, NamedNode | Literal>;
/**
 * A {@link QueryAdapter} that stores data in a database through a sparql endpoint.
 */
export declare class SparqlQueryAdapter implements QueryAdapter {
    private readonly sparqlClient;
    private readonly sparqlGenerator;
    private readonly setTimestamps;
    constructor(options: SparqlQueryAdapterOptions);
    executeRawQuery<T extends RawQueryResult>(query: string): Promise<T[]>;
    executeRawEntityQuery(query: string, frame?: Frame): Promise<GraphObject>;
    find(options?: FindOneOptions): Promise<Entity | null>;
    private sparqlSelect;
    findBy(where: FindOptionsWhere): Promise<Entity | null>;
    findAll(options?: FindAllOptions): Promise<Entity[]>;
    private groupByKey;
    private sparqlSelectGroup;
    findAllBy(where: FindOptionsWhere): Promise<Entity[]>;
    exists(where: FindOptionsWhere): Promise<boolean>;
    private sparqlAsk;
    count(where?: FindOptionsWhere): Promise<number>;
    private sparqlCountSelect;
    private sparqlSelectGraph;
    save(entity: Entity): Promise<Entity>;
    save(entities: Entity[]): Promise<Entity[]>;
    destroy(entity: Entity): Promise<Entity>;
    destroy(entities: Entity[]): Promise<Entity[]>;
    destroyAll(): Promise<void>;
    private executeSparqlSelectAndGetData;
    private executeSparqlUpdate;
    private executeAskQueryAndGetResponse;
    private executeSelectCountAndGetResponse;
}
